@@ -74,10 +74,7 @@ SOFTWARE.
 /*
  * Standard integers
  */
-#if defined (__SVR4) && defined (__sun)
-#  include <sys/int_types.h>
-#  include <stdint.h>
-#elif defined (__OpenBSD__) || defined (__sgi)
+#if defined (_SVR4) || defined (SVR4) || defined (__OpenBSD__) || defined (_sgi)
 #  include <inttypes.h>
 #elif defined (_MSC_VER)
 typedef __int8 int8_t;
